/* $Id: state_imp.h,v 2.9 2011-09-25 18:37:09 hurtta Exp $ */

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.9 $
 *
 *  Author: Kari Hurtta <hurtta+elm@siilo.FMI.FI> (was hurtta+elm@posti.FMI.FI)
 *       or Kari Hurtta <elm@elmme-mailer.org>
 *****************************************************************************/




/* ----------------------------------------------------------------------- */


typedef void init_so_func P_((struct out_state *s)); 
typedef void dest_so_func P_((struct out_state *s));

typedef int  seekable_so_func P_((struct out_state *s));
typedef FILE * FILE_so_func P_((struct out_state *s));
typedef int  seek_so_func P_((struct out_state *s, long pos));
typedef long ftell_so_func P_((struct out_state *s));
typedef void flush_filter_so_func P_((struct out_state *s, int munge_eoln,
				      int pg_flags));

enum state_policy { state_policy_putc_use_filter = 1,
		    state_policy_flush_via_put,
		    state_policy_support_pg_flags
};

typedef int policy_so_func P_((struct out_state *s, 
			       enum state_policy question));

typedef int  putc_so_func P_((struct out_state *s, int ch));
typedef int  put_so_func P_((struct out_state *s, const char *string, int len));

#if NOTUSED
/* Returns 0 if not supported, flush_filter is already called
   should be called on beginning of line
   flags should be zeroed implicity on end of line
 */
typedef int set_line_pg_flags_func P_(( struct out_state *ptr, int pg_flags));
#endif

typedef int ferror_so_func P_((struct out_state *s));

#define OUT_STATE_type_magic 0xEB05

struct out_state_type {
    unsigned short magic;     /* OUT_STATE_type_magic */
    
    init_so_func     * init_so_it;
    dest_so_func     * dest_so_it;
    seekable_so_func * seekable_so_it;
    FILE_so_func     * FILE_so_it;
    seek_so_func     * seek_so_it;
    ftell_so_func    * ftell_so_it;
    flush_filter_so_func * flush_filter_so_it;
    policy_so_func   * policy_so_it;
    putc_so_func     * putc_so_it;
    put_so_func      * put_so_it;
    ferror_so_func   * ferror_so_it;
};

/* This only registers pointer, struct is not copied */
void register_state_out_type P_((struct out_state_type *T));


/* ---------------------------------------------------------------------- */

#define STATE_out_magic	     0xEB00

struct out_state {
    unsigned short magic;     /* STATE_out_magic */

    struct out_state_type   *  state_type;

    unsigned int EOLN_is_CRLF : 1;       /* 0 == \n   is EOLN
					    1 == \r\n is EOLN */

    charset_t    * display_charset;      /* output charset -- vector
					  * of possible sets 
					  */
    charset_t       filter;
    struct string * filter_line;       /* input charset */
    int             pg_flags;          /* pager flags   pg_BOLD, and so on */

    union {
	void                  * dummy;
	struct out_state_file * file;
	struct stringbuffer   * buffer;
	struct dirstate_out   * dir;
    } u;
};


/* ---------------------------------------------------------------------- */

extern void state_panic P_((char *,int,char *, char *));


/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
