static char rcsid[] = "@(#)$Id: state_in.c,v 2.2 2011-03-06 19:52:58 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.2 $   $State: Exp $
 *
 *  Modified by: Kari Hurtta <hurtta+elm@siilo.FMI.FI>
 *                       (was hurtta+elm@posti.FMI.FI) 
 *           or  Kari Hurtta <elm@elmme-mailer.org>
 *
 *
 *  
 *  Some code copied from lib/state.c. It have following note:
 *
 *     Initially written by: Michael Elkins <elkins@aero.org>, 1995
 *****************************************************************************/

#include "elm_defs.h"
#include "state_imp.h"

DEBUG_VAR(DebugIO,__FILE__,"stateio");


#define IN_STATE_type_magic  0xEA02

typedef void init_si_type_f P_((struct in_state *)); /* Prototype */
typedef void dest_si_type_f P_((struct in_state *)); /* Prototype */
typedef int  getc_si_type_f P_((struct in_state *)); /* Prototype */
typedef int  ungetc_si_type_f P_((int,struct in_state *)); /* Prototype */
typedef char *gets_si_type_f P_((char *, int, struct in_state *));
typedef int getl_si_type_f P_((char *, int, struct in_state *));

struct in_state_type {
    unsigned short magic;   /* IN_STATE_type_magic */

    init_si_type_f     * init_si_type;
    dest_si_type_f     * dest_si_type;
    int                  seekable;
    getc_si_type_f     * getc_si_type;
    ungetc_si_type_f   * ungetc_si_type;
    gets_si_type_f     * gets_si_type;
    getl_si_type_f     * getl_si_type;
};



/* ------------------------------------------------------ */

typedef struct {
    FILE *fpin;
} in_state_file;

#define STATE_in_magic       0xEA00

struct in_state {
    unsigned short magic;       /* STATE_in_magic */
    
    struct in_state_type   * state_type;

    union {
	in_state_file    file;
    } u;

};


struct in_state * new_in_state(m)
     struct in_state_type * m;
{
    struct in_state * res = safe_malloc(sizeof (*res));
    
    /* defined in hdrs/defs.h */
    bzero((void *)res,sizeof (*res));
    
    if (IN_STATE_type_magic != m->magic)
	state_panic(__FILE__,__LINE__,"new_in_state","Bad magic number");

    res->state_type = m;

    res->magic = STATE_in_magic;  
    
    m->init_si_type(res);


    return res;
}

void free_in_state(ptr) 
     struct in_state **ptr;
{

    if (STATE_in_magic != (*ptr)->magic)
	state_panic(__FILE__,__LINE__,"free_in_state",
		    "Bad state magic number");

    if (IN_STATE_type_magic != (*ptr)->state_type->magic)
	state_panic(__FILE__,__LINE__,"free_in_state",
		    "Bad state type magic number");
	
    (*ptr)->state_type->dest_si_type(*ptr);

    (*ptr)->state_type = NULL;
    (*ptr)->magic = 0;   /* Invalidate */

    free(*ptr);
    *ptr = NULL;
}


int in_state_seekable (s)
     struct in_state *s;
{

    if (STATE_in_magic != s->magic)
	state_panic(__FILE__,__LINE__,"in_state_seekable",
		    "Bad state magic number");

    if (IN_STATE_type_magic != s->state_type->magic)
	state_panic(__FILE__,__LINE__,"in_state_seekable",
		    "Bad state type magic number");

    return s->state_type->seekable;
}

int state_getc (s)
     struct in_state *s;
{
    int r = EOF;

    DPRINT(DebugIO,30,(&DebugIO,
		       "state_getc(s=%p)\n",s));

    if (STATE_in_magic != s->magic)
	state_panic(__FILE__,__LINE__,"state_getc",
		    "Bad state magic number");

    if (IN_STATE_type_magic != s->state_type->magic)
	state_panic(__FILE__,__LINE__,"state_getc",
		    "Bad state type magic number");

    r = s->state_type->getc_si_type(s);
        
    DPRINT(DebugIO,30,(&DebugIO,
		       "state_getc=%d\n",
		       r));
    
    return r;
}

int state_ungetc (c,s)
     int c;
     struct in_state *s;
{
    int r = EOF;

    DPRINT(DebugIO,30,(&DebugIO,
		       "state_ungetc(s=%p)\n",s));


    if (STATE_in_magic != s->magic)
	state_panic(__FILE__,__LINE__,"state_ungetc",
		    "Bad state magic number");

    if (IN_STATE_type_magic != s->state_type->magic)
	state_panic(__FILE__,__LINE__,"state_ungetc",
		    "Bad state type magic number");


    r = s->state_type->ungetc_si_type(c,s);

    DPRINT(DebugIO,30,(&DebugIO,
		       "state_ungetc=%d\n",
		       r));
    
    return r;
}


char *state_gets (dest, length, s)
     char *dest;
     int length;
     struct in_state *s;
{
    char * r = NULL;

    DPRINT(DebugIO,30,(&DebugIO,
		       "state_gets(s=%p)\n",s));

    if (STATE_in_magic != s->magic)
	state_panic(__FILE__,__LINE__,"state_gets",
		    "Bad state magic number");

    if (IN_STATE_type_magic != s->state_type->magic)
	state_panic(__FILE__,__LINE__,"state_gets",
		    "Bad state type magic number");

    r = s->state_type->gets_si_type(dest,length,s);
    
    DPRINT(DebugIO,30,(&DebugIO,
		       "state_gets=%p\n",
		       r));
    
    return r;
}

int state_getl (dest, length, s)
     char *dest;
     int length;
     struct in_state *s;
{
    int r = 0;

    DPRINT(DebugIO,30,(&DebugIO,
		       "state_getl(s=%p) -- magic=%d\n",s));
  
    if (STATE_in_magic != s->magic)
	state_panic(__FILE__,__LINE__,"state_getl",
		    "Bad state magic number");

    if (IN_STATE_type_magic != s->state_type->magic)
	state_panic(__FILE__,__LINE__,"state_getl",
		    "Bad state type magic number");

    r = s->state_type->getl_si_type(dest,length,s);

    DPRINT(DebugIO,30,(&DebugIO,
		       "state_getl=%d\n",
		       r));
    
    return r;
}

/* ----------------------------------------------------------------- */

#if ANSI_C
#define S_(x) static x;
#else
#define S_(x)
#endif


S_(init_si_type_f init_si_file)
static void init_si_file P_((struct in_state *)); /* Prototype */
static void init_si_file(s)
     struct in_state *s;
{
    s->u.file.fpin = NULL;
}

S_(dest_si_type_f dest_si_file)
static void dest_si_file P_((struct in_state *)); /* Prototype */
static void dest_si_file(s)
     struct in_state *s;
{
    s->u.file.fpin = NULL;
}


void set_in_state_file (F, s)
     FILE *F;
     struct in_state *s;
{
    if (STATE_in_magic != s->magic)
	state_panic(__FILE__,__LINE__,"set_in_state_file","Bad magic number");
    
    if (STATE_in_file != s->state_type)
	state_panic(__FILE__,__LINE__,"set_in_state_file","Bad state type");

    if (s->u.file.fpin)
	state_panic(__FILE__,__LINE__,"set_in_state_file","Already called");
    
    s->u.file.fpin = F;
}

int in_state_fseek (s,pos)
     struct in_state *s;
     long pos;
{
    if (STATE_in_magic != s->magic)
	state_panic(__FILE__,__LINE__,"in_state_fseek",
		    "Bad state magic number");

    if (IN_STATE_type_magic != s->state_type->magic)
	state_panic(__FILE__,__LINE__,"in_state_fseek",
		    "Bad state type magic number");
    
    if (s->u.file.fpin == NULL)
	state_panic(__FILE__,__LINE__,"in_state_fseek","NULL file pointer");
    
    return fseek(s->u.file.fpin,pos,SEEK_SET);
}

long in_state_ftell (s)
     struct in_state *s;
{
    if (STATE_in_magic != s->magic)
	state_panic(__FILE__,__LINE__,"in_state_ftell",
		    "Bad state magic number");

    if (IN_STATE_type_magic != s->state_type->magic)
	state_panic(__FILE__,__LINE__,"in_state_ftell",
		    "Bad state type magic number");
    
    if (s->u.file.fpin == NULL)
	state_panic(__FILE__,__LINE__,"in_state_ftell","NULL file pointer");
    
    return ftell(s->u.file.fpin);
}

S_(getc_si_type_f getc_si_file)
static int getc_si_file P_((struct in_state *)); /* Prototype */
static int getc_si_file(s)
     struct in_state *s;
{    
    if (s->u.file.fpin == NULL)
	state_panic(__FILE__,__LINE__,"getc_si_file","NULL filepointer");

    return fgetc (s->u.file.fpin);
}

S_(ungetc_si_type_f ungetc_si_file)
static int ungetc_si_file P_((int,struct in_state *)); /* Prototype */
static int ungetc_si_file (c,s)
     int c;
     struct in_state *s;
{
    if (s->u.file.fpin == NULL)
	state_panic(__FILE__,__LINE__,"ungetc_si_file","NULL filepointer");

    return ungetc(c,s->u.file.fpin);
}


S_(gets_si_type_f gets_si_file)
static char *gets_si_file P_((char *, int, struct in_state *));
static char *gets_si_file (dest, length, s)
     char *dest;
     int length;
     struct in_state *s;
{
    if (s->u.file.fpin == NULL)
	state_panic(__FILE__,__LINE__,"gets_si_file","NULL filepointer");

    return (fgets (dest, length, s->u.file.fpin));
}

S_(getl_si_type_f getl_si_file)
static int getl_si_file P_((char *, int, struct in_state *));
static int getl_si_file (dest, length, s)
     char *dest;
     int length;
     struct in_state *s;
{
    if (s->u.file.fpin == NULL)
	state_panic(__FILE__,__LINE__,"getl_si_file","NULL filepointer");
    
    return (mail_gets (dest, length, s->u.file.fpin));
}


struct in_state_type STATE_in_file_type = {
    IN_STATE_type_magic,

    init_si_file,
    dest_si_file,
    1,
    getc_si_file,
    ungetc_si_file,
    gets_si_file,
    getl_si_file
};


/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
